#ifndef CG_SOLVER_HH
#define CG_SOLVER_HH

#include <dune/solvers/solvers/iterativesolver.hh>
#include <dune/solvers/iterationsteps/lineariterationstep.hh>
#include <dune/solvers/norms/norm.hh>

/** \brief A conjugate gradient solver 
 *
 */
template <class MatrixType, class VectorType>
class CGSolver : public IterativeSolver<VectorType>
{
    static const int blocksize = VectorType::block_type::dimension;

public:     

    /** \brief Constructor taking all relevant data */
        
    CGSolver(const MatrixType* matrix,
             VectorType* x,
             VectorType* rhs,
             LinearIterationStep<MatrixType,VectorType>* preconditioner,
             int maxIterations,
             double tolerance,
             Norm<VectorType>* errorNorm,
             NumProc::VerbosityMode verbosity,
             bool useRelativeError=true)
      : IterativeSolver<VectorType>(tolerance,maxIterations,verbosity,useRelativeError),
          matrix_(matrix), x_(x), rhs_(rhs),
          preconditioner_(preconditioner), errorNorm_(errorNorm)
    {}
    
    /** \brief Loop, call the iteration procedure
     * and monitor convergence */
    virtual void solve();
    
    /** \brief Checks whether all relevant member variables are set
     * \exception SolverError if the iteration step is not set up properly
     */
    virtual void check() const;

    const MatrixType* matrix_;

    VectorType* x_;

    VectorType* rhs_;
    
    //! The iteration step used by the algorithm
    LinearIterationStep<MatrixType,VectorType>* preconditioner_;
    
    //! The norm used to measure convergence
    Norm<VectorType>* errorNorm_;
    
};

#include "cgsolver.cc"

#endif
